//**************************************************************************
//*                     This file is part of the                           *
//*                MMC - Mpxplay Multimedia Commander                      *
//*                   The source code of MMC is                            *
//*        (C) copyright 1998-2016 by PDSoft (Attila Padar)                *
//*                http://mpxplay.sourceforge.net                          *
//*                  email: mpxplay@freemail.hu                            *
//**************************************************************************
//*  This program is distributed in the hope that it will be useful,       *
//*  but WITHOUT ANY WARRANTY; without even the implied warranty of        *
//*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                  *
//*  Please contact with the author (with me) if you want to use           *
//*  or modify this source.                                                *
//**************************************************************************
//function: video widget handling

//#define MPXPLAY_USE_DEBUGF 1
#define DISPQT_DEBUG_OUTPUT NULL //stdout
#define DISPQT_DEBUG_ERROR stderr
#define DISPQT_DEBUGOUT_FFMPEG NULL
#define DISPQT_DEBUGOUT_RENDER NULL // stdout
#define DISPQT_DEBUGOUT_SUBTITLE NULL // stdout
#define DISPQT_DEBUGOUT_CROP NULL // stdout
#define DISPQT_DEBUGOUT_ACROP NULL // stdout

#include <QtGui>
#include <QtWidgets>
#include <QtGui/QPixMap>
#include <QtPlatformHeaders/QWindowsWindowFunctions.h>
#include "moc_video_qt.h"
#include "moc_mainwindow.h"

#define DISPQT_VIDEO_AR_SCALE 1000
#define DISPQT_VIDEO_ROUND_VALUE 1
#define DISPQT_VIDEO_ROUND_MASK (~1)

#define DISPQT_VIDEO_AUTOCROP_COUNTER_OPEN     2  // at opening a new video
#define DISPQT_VIDEO_AUTOCROP_COUNTER_REFRESH  1  // at seeking or at window resize

#ifdef MPXPLAY_LINK_ORIGINAL_FFMPEG

#if (DISPQT_USE_OPENGL_WIDGET == 2)
FFMpegVideoWidget::FFMpegVideoWidget(MainWindow *main_window, QWidget *parent) : QOpenGLWidget(parent)
#else
FFMpegVideoWidget::FFMpegVideoWidget(MainWindow *main_window, QWidget *parent) : QWidget(parent)
#endif
{
	struct mmc_dispqt_config_s *gcfg = main_window->gui_config;
	mpxplay_debugf(DISPQT_DEBUGOUT_FFMPEG,"FFMpegVideoWidget INIT");
	this->main_window = main_window;
	this->parent_widget = parent;

	this->sws_src_width = this->sws_src_height = 0;
	this->sws_srccrop_x = this->sws_srccrop_y = 0;
	this->sws_srccrop_width = this->sws_srccrop_height = 0;
	this->sws_crop_w = this->sws_crop_h = 0;
	this->sws_crop_x = this->sws_crop_y = 0;
	this->sws_src_pix_fmt = AV_PIX_FMT_NONE;
	this->last_video_crop_type = gcfg->video_crop_type;
	this->last_video_zoom_min_val = this->last_video_zoom_ti_val = 0;
	this->last_video_render_type = gcfg->video_renderer_type;

	this->sws_dst_width = this->width();
	this->sws_dst_height = this->height();
	this->video_zoom_globalpos_x = this->sws_dst_width / 2;
	this->video_zoom_globalpos_y = this->sws_dst_height / 2;
	this->sws_dst_pos_x = this->sws_dst_pos_y = 0;
	this->sws_autoxcrop_framecounter = 0;

	//cleared in videowidget_ffmpeg_videoctx_clear (not needed)
#ifdef USE_SWS_IMAGE_SCALING
	this->ffmpeg_videoout_frame = NULL;
#endif

	// cleared in videowidget_ffmpeg_swsctx_clear too (not needed)
	this->ffmpeg_swsframe_hasdata = this->ffmpeg_swsframe_clearpic = false;
	this->ffmpeg_swsframe_cropreset = DISPQT_VIDEO_AUTOCROP_COUNTER_OPEN;
	this->ffmpeg_sws_frame = NULL;
	this->sws_buffer = NULL;
	this->sws_ctx = NULL;
	this->sub_title_infos = NULL;

#if (DISPQT_USE_OPENGL_WIDGET == 1)
//	this->windowHandle()->setSurfaceType(QWindow::OpenGLSurface);
	this->video_opengl_context = new QOpenGLContext(this);
	this->video_opengl_context->create();
#elif (DISPQT_USE_OPENGL_WIDGET == 2)
	QWindowsWindowFunctions::setHasBorderInFullScreen(main_window->windowHandle(), true);
#endif

	this->videowidget_ffmpeg_videoctx_init();
	this->videowidget_ffmpeg_swsctx_clear();

	connect(this, SIGNAL(signal_ffmpeg_swctx_resize(int,int,bool)), this, SLOT(videowidget_ffmpeg_swctx_resize(int,int,bool)));
	connect(this, SIGNAL(signal_ffmpeg_swctx_move(int,int)), this, SLOT(videowidget_ffmpeg_swctx_move(int,int)));
	connect(this, SIGNAL(signal_ffmpeg_swctx_zoom(int,int,int)), this, SLOT(videowidget_ffmpeg_swctx_zoom(int,int,int)));

	//this->video_renderer_evr = new EVRCustomPresenter(NULL);
	//this->video_sample_scheduler_evr = new Scheduler(this->video_renderer_evr);
	this->video_decoder_worker = new FFMpegVideoDecoderWorker(main_window, this);
	QObject::connect(this->video_decoder_worker, SIGNAL(signal_ffmpeg_frame_display(int, unsigned int, AVFrame *, ffmpegvideo_subtitle_info_s *)), this, SLOT(videowidget_ffmpeg_frame_display(int, unsigned int, AVFrame *, ffmpegvideo_subtitle_info_s *)));
	QObject::connect(&this->video_decoder_timer, SIGNAL(timeout()), this->video_decoder_worker, SLOT(TimedEvent()));
	//QObject::connect(&this->video_decoder_thread, SIGNAL(finished()), this->video_decoder_worker, SLOT(deleteLater()));

	this->video_decoder_timer.start(DISPQT_FFMPEG_WORKERTHREAD_REFRESH_MS);
	this->video_decoder_timer.moveToThread(&this->video_decoder_thread);
	this->video_decoder_worker->moveToThread(&this->video_decoder_thread);
	this->video_decoder_thread.start();
}

FFMpegVideoWidget::~FFMpegVideoWidget()
{
	this->mutex_video.tryLock(DISPQT_VIDEO_MUTEX_TIMEOUT);
	this->video_decoder_timer.stop();
	if(this->video_decoder_worker){
		this->video_decoder_thread.quit();
		delete this->video_decoder_worker;
		this->video_decoder_worker = NULL;
	}
	videowidget_ffmpeg_swsctx_close();
	videowidget_ffmpeg_videoctx_close();
	this->mutex_video.unlock();
}

// --------------------------------------------------------------------------------------------------------------------

int FFMpegVideoWidget::videowidget_ffmpeg_videoctx_init(void)
{
	int retcode = -1;
	this->videowidget_ffmpeg_videoctx_clear();
#ifdef USE_SWS_IMAGE_SCALING
	this->ffmpeg_videoout_frame = av_frame_alloc();
	if(!this->ffmpeg_videoout_frame)
		goto err_out_init;
#endif
	retcode = MPXPLAY_ERROR_OK;
err_out_init:
	return retcode;
}
void FFMpegVideoWidget::videowidget_ffmpeg_videoctx_close(void)
{
#ifdef USE_SWS_IMAGE_SCALING
	if(this->ffmpeg_videoout_frame)
		av_frame_free(&this->ffmpeg_videoout_frame);
#endif
	this->videowidget_ffmpeg_videoctx_clear();
}

void FFMpegVideoWidget::videowidget_ffmpeg_videoctx_clear(void)
{
#ifdef USE_SWS_IMAGE_SCALING
	this->ffmpeg_videoout_frame = NULL;
#endif
	this->video_aspect_ratio.num = 1;
	this->video_aspect_ratio.den = 0;
}

void FFMpegVideoWidget::videowidget_ffmpeg_swctx_resize(int window_width, int window_height, bool do_repaint)
{
	struct mmc_dispqt_config_s *gcfg = this->main_window->gui_config;
	this->mutex_video.tryLock(DISPQT_VIDEO_MUTEX_TIMEOUT);
	if(window_width < 0)
		window_width = this->width();
	if(window_height < 0)
		window_height = this->height();
	if(this->last_video_crop_type != gcfg->video_crop_type){
		this->last_video_crop_type = gcfg->video_crop_type;
		if(!this->ffmpeg_swsframe_cropreset)
			this->ffmpeg_swsframe_cropreset = DISPQT_VIDEO_AUTOCROP_COUNTER_REFRESH;
		this->sws_autoxcrop_framecounter = 0;
		if((gcfg->video_crop_type == DISPQT_VIDEOSCRCROPTYPE_AUTO) && this->ffmpeg_sws_frame)
			this->videowidget_ffmpeg_swsctx_crop_auto_detect(this->ffmpeg_sws_frame->data[0]);
	}
#ifdef USE_SWS_IMAGE_SCALING
	this->videowidget_ffmpeg_swsctx_init(window_width, window_height);
	if(this->sws_ctx && this->ffmpeg_sws_frame && this->ffmpeg_videoout_frame && this->ffmpeg_videoout_frame->data[0] && this->ffmpeg_videoout_frame->linesize){
		sws_scale(this->sws_ctx, (uint8_t const * const *)&this->ffmpeg_videoout_frame->data[0],
			this->ffmpeg_videoout_frame->linesize, 0, this->sws_src_height,
			&this->ffmpeg_sws_frame->data[0], &this->ffmpeg_sws_frame->linesize[0]);
		this->ffmpeg_swsframe_hasdata = true;
	}
#else
	this->videowidget_ffmpeg_swsctx_calculate(window_width, window_height);
#endif
	this->mutex_video.unlock();
	if(do_repaint)
		this->repaint();
}

void FFMpegVideoWidget::videowidget_ffmpeg_swctx_move(int move_x, int move_y)
{
	struct mmc_dispqt_config_s *gcfg = this->main_window->gui_config;
	int max_crop_x, max_crop_y, prev_crop_x = this->sws_crop_x, prev_crop_y = this->sws_crop_y;

	move_x = move_x * DISPQT_CONFIG_VIDEOZOOM_SCALE / gcfg->video_zoom_value;  // scale moving to zoom rate
	max_crop_x = this->sws_srccrop_width - this->sws_crop_w;
	if(max_crop_x > 0)
	{
		this->sws_crop_x += move_x;
		if(this->sws_crop_x < this->sws_srccrop_x)
			this->sws_crop_x = this->sws_srccrop_x;
		else if(this->sws_crop_x > (this->sws_srccrop_x + max_crop_x))
			this->sws_crop_x = this->sws_srccrop_x + max_crop_x;
	}

	move_y = move_y * DISPQT_CONFIG_VIDEOZOOM_SCALE / gcfg->video_zoom_value;
	max_crop_y = this->sws_srccrop_height - this->sws_crop_h;
	if(max_crop_y > 0)
	{
		this->sws_crop_y += move_y;
		if(this->sws_crop_y < this->sws_srccrop_y)
			this->sws_crop_y = this->sws_srccrop_y;
		else if(this->sws_crop_y > (this->sws_srccrop_y + max_crop_y))
			this->sws_crop_y = this->sws_srccrop_y + max_crop_y;
	}

	// reconfig zoom center after move
	this->video_zoom_globalpos_x = this->sws_dst_width / 2;
	this->video_zoom_globalpos_y = this->sws_dst_height / 2;
	if(!this->isFullScreen()){
		this->video_zoom_globalpos_x += QWidget::mapToGlobal(this->pos()).x();
		this->video_zoom_globalpos_y += QWidget::mapToGlobal(this->pos()).y();
	}

	if((prev_crop_x != this->sws_crop_x) || (prev_crop_y = this->sws_crop_y)){
		mpxplay_debugf(DISPQT_DEBUGOUT_FFMPEG, "MOVE cx:%d cy:%d", this->sws_crop_x, this->sws_crop_y);
		// TODO: sw crop (not QPainter based) for EVR
		this->repaint();
	}
}

void FFMpegVideoWidget::videowidget_ffmpeg_swctx_zoom(int globalpos_x, int globalpos_y, int direction)
{
	struct mmc_dispqt_config_s *gcfg = this->main_window->gui_config;
	int i, zoom_val, center_values[2];

	if((this->last_video_zoom_min_val <= 0) || (this->last_video_zoom_ti_val <= 0) || (this->sws_srccrop_width <= 0) || (this->sws_srccrop_height <= 0) || (gcfg->video_zoom_value <= 0))
		return;

	zoom_val = gcfg->video_zoom_value + direction * DISPQT_CONFIG_VIDEOZOOM_STEP * gcfg->video_zoom_value / DISPQT_CONFIG_VIDEOZOOM_SCALE;

	center_values[0] = DISPQT_CONFIG_VIDEOZOOM_CENTER;
	center_values[1] = this->last_video_zoom_ti_val;
	mpxplay_debugf(DISPQT_DEBUGOUT_CROP, "swctx_zoom cv:%d zv:%d mv:%d", gcfg->video_zoom_value, zoom_val, this->last_video_zoom_min_val);

	if(zoom_val < this->last_video_zoom_min_val)
		zoom_val = this->last_video_zoom_min_val;
	else if(zoom_val > DISPQT_CONFIG_VIDEOZOOM_MAX)
		zoom_val = DISPQT_CONFIG_VIDEOZOOM_MAX;

	for(i = 0; i < (sizeof(center_values) / sizeof(center_values[0])); i++)
	{
		const int center = center_values[i];
		//mpxplay_debugf(DISPQT_DEBUGOUT_CROP, "swctx_zoom i:%d c:%d", i, center);
		if(((gcfg->video_zoom_value > center) && (zoom_val <= (center + center / 10))) || ((gcfg->video_zoom_value < center) && (zoom_val >= (center - center / 10)))){
			zoom_val = center;
			mpxplay_debugf(DISPQT_DEBUGOUT_CROP, "swctx_zoom i:%d c:%d", i, center);
			break;
		}
	}

	mpxplay_debugf(DISPQT_DEBUGOUT_CROP, "swctx_zoom zt:%d zv:%d mv:%d", zoom_type, zoom_val, this->last_video_zoom_min_val);

	if(gcfg->video_zoom_value != zoom_val)
	{
		gcfg->video_zoom_type = DISPQT_VIDEOFULLSCREENZOOMTYPE_MANUAL;
		gcfg->video_zoom_value = zoom_val;
		if(globalpos_x >= 0)
			this->video_zoom_globalpos_x = globalpos_x;
		if(globalpos_y >= 0)
			this->video_zoom_globalpos_y = globalpos_y;
		this->videowidget_ffmpeg_swctx_resize(-1,-1,true);
	}
	this->main_window->qt_video_player->video_mouse_cursor_show(true);
}

void FFMpegVideoWidget::resizeEvent(QResizeEvent *e)
{
	this->videowidget_ffmpeg_swctx_resize(e->size().width(), e->size().height(), false);
#if (DISPQT_USE_OPENGL_WIDGET == 2)
	QOpenGLWidget::resizeEvent(e);
#else
	QWidget::resizeEvent(e);
#endif
}

void FFMpegVideoWidget::videowidget_ffmpeg_render_subtitle(QPainter *painter, ffmpegvideo_subtitle_info_s *subtitle_infos)
{
	struct mmc_dispqt_config_s *gcfg = this->main_window->gui_config;
	int i, scale, charpixelsize, x, y, cy, cx;
	unsigned int flags;
#ifdef DISPQT_FFMPEG_TEST_SPEED
	mpxp_int64_t render_time = pds_gettimem();
#endif

	if(!painter || !subtitle_infos)
		return;

	if(subtitle_infos->last_size_x && subtitle_infos->last_size_y)
	{
		// clear black subtitle areas
		if(subtitle_infos->last_render_x < this->sws_dst_pos_x)
			painter->fillRect(0, 0, this->sws_dst_pos_x, this->height(), Qt::black); // left side
		cx = this->sws_dst_pos_x + this->sws_dst_width;
		if((this->width() > cx) && ((subtitle_infos->last_render_x + subtitle_infos->last_size_x) > cx))
			painter->fillRect(cx, 0, this->width() - cx, this->height(), Qt::black);  // right side
		if(subtitle_infos->last_render_y < this->sws_dst_pos_y)
			painter->fillRect(0, 0, this->width(), this->sws_dst_pos_y, Qt::black);   // top side
		cy = this->sws_dst_pos_y + this->sws_dst_height;
		if((this->height() > cy) && ((subtitle_infos->last_render_y + subtitle_infos->last_size_y) > cy))
			painter->fillRect(0, cy, this->width(), this->height() - cy, Qt::black);  // bottom side
	}

	subtitle_infos->last_render_x = subtitle_infos->last_render_y = subtitle_infos->last_size_x = subtitle_infos->last_size_y = 0;

	if(!subtitle_infos->nb_lines)
		return;

	scale = 100 * this->height() / QApplication::desktop()->screenGeometry().height();
	if(scale < 5)
		scale = 5;
	else if(scale > 100)
		scale = 100;

	if(gcfg->font_text_name[DISPQT_FONTTEXTID_SUBTITLE])
		this->font_subtitle.fromString(QString::fromUtf8(gcfg->font_text_name[DISPQT_FONTTEXTID_SUBTITLE])); // TODO: move out from here
	charpixelsize = this->font_subtitle.pointSize();
	charpixelsize = charpixelsize * scale / 100;
	this->font_subtitle.setPointSize(charpixelsize);
	painter->setFont(this->font_subtitle);

	y = this->height() - subtitle_infos->nb_lines * charpixelsize * 3 / 2 - charpixelsize * 2;
	if(y < 0)
		y = 0;
	subtitle_infos->last_render_y = y;
	flags = 0;

	for(i = 0; i < subtitle_infos->nb_lines; i++)
	{
		QFontMetrics fm(this->font_subtitle);
		int textpixellen = fm.width(QString::fromUtf8(subtitle_infos->line_texts[i]));
		if(textpixellen <= 0)
			continue;
		x = this->width() / 2 - textpixellen / 2;
		if(x < 0)
			x = 0;
		if(flags != subtitle_infos->line_flags[i]){
			flags = subtitle_infos->line_flags[i];
			this->font_subtitle.setItalic((flags & INFFMPEG_ASS_DECODED_FLAG_ITALIC)? true : false);
			painter->setFont(this->font_subtitle);
		}
		painter->setPen(Qt::black);
		painter->drawText(x + 2, y + 2, QString::fromUtf8(subtitle_infos->line_texts[i])); // shadow

		painter->setPen(Qt::white);
		painter->drawText(x, y, QString::fromUtf8(subtitle_infos->line_texts[i])); // white text

		//mpxplay_debugf(DISPQT_DEBUGOUT_SUBTITLE, "SUBTITLE drawText: w:%d cs:%d x:%d y:%d l:%d cy:%d %s", this->width(), charpixelsize, x, y, i, cy, subtitle_infos->line_texts[i]);
		y += charpixelsize * 3 / 2;
		subtitle_infos->last_render_x = (i == 0)? x : min(x, subtitle_infos->last_render_x);
		subtitle_infos->last_size_x = max(subtitle_infos->last_size_x, textpixellen);
	}
	subtitle_infos->last_size_y = y - subtitle_infos->last_render_y;
#ifdef DISPQT_FFMPEG_TEST_SPEED
	render_time = pds_gettimem() - render_time;
	if(render_time > 0){
		//mpxplay_debugf(DISPQT_DEBUGOUT_RENDER, "videowidget_ffmpeg_render_subtitle TIME %llu", render_time);
	}
#endif
}

void FFMpegVideoWidget::paintEvent(QPaintEvent *event)
{
#if (DISPQT_USE_OPENGL_WIDGET == 1)
	this->video_opengl_context->makeCurrent(this->video_opengl_context->surface());
	QOpenGLPaintDevice device(size() * devicePixelRatio());
	QPainter p(&device);
//#elif (DISPQT_USE_OPENGL_WIDGET == 2)
//    QOpenGLPaintDevice device(size() * devicePixelRatio());
//    QPainter p(&device);
#else
	QPainter p;
#endif
	p.begin(this);
	this->mutex_video.tryLock(DISPQT_VIDEO_MUTEX_TIMEOUT);
	if(this->ffmpeg_swsframe_hasdata && this->sws_ctx && this->ffmpeg_sws_frame && this->ffmpeg_sws_frame->data[0] && this->ffmpeg_sws_frame->linesize[0]){
		if(this->ffmpeg_swsframe_clearpic){
			p.fillRect(0,0,this->width(),this->height(),Qt::black);
			this->ffmpeg_swsframe_clearpic = false;
			mpxplay_debugf(DISPQT_DEBUGOUT_RENDER, "paintEvent CLEARPIC");
		}
#ifdef USE_SWS_IMAGE_SCALING
		QImage image(this->ffmpeg_sws_frame->data[0], this->sws_dst_width, this->sws_dst_height, QImage::Format_RGB32);
		p.drawPixmap(this->sws_dst_pos_x, this->sws_dst_pos_y, QPixmap::fromImage(image)); // TODO: requires extra crop function
#else
		p.setRenderHint(QPainter::SmoothPixmapTransform, true);
		QImage image(this->ffmpeg_sws_frame->data[0], this->sws_src_width, this->sws_src_height, QImage::Format_RGB32);
		p.drawImage(QRectF(this->sws_dst_pos_x, this->sws_dst_pos_y, this->sws_dst_width, this->sws_dst_height), image,
				QRectF(this->sws_crop_x, this->sws_crop_y, this->sws_crop_w, this->sws_crop_h), Qt::NoFormatConversion);
		//p.drawImage(QRect(this->sws_dst_pos_x, this->sws_dst_pos_y, this->sws_dst_width, this->sws_dst_height), image);
		this->videowidget_ffmpeg_render_subtitle(&p, this->sub_title_infos);
#endif
	}else{
		p.fillRect(0,0,this->width(),this->height(),Qt::black);
		mpxplay_debugf(DISPQT_DEBUGOUT_RENDER, "paintEvent CLEAR2");
	}
	p.end();
#if (DISPQT_USE_OPENGL_WIDGET == 1)
	this->video_opengl_context->swapBuffers(this->video_opengl_context->surface());
//#elif (DISPQT_USE_OPENGL_WIDGET == 2)
//	QOpenGLWidget::paintEvent(event);
#endif
	this->mutex_video.unlock();
}

//-------------------------------------------------------------------------------------------------------------------------------------
static int videowidget_ffmpeg_swsctx_get_custom_ar(char *custom_str, int scale_value, int default_value)
{
	int ar_value = default_value;
	char *ar_separator = pds_strchr(custom_str, DISPQT_CONFIG_VIDEOAR_DIVISORCHAR);
	if(ar_separator){
		int ar_custom_x = pds_atol(custom_str);
		int ar_custom_y = pds_atol(ar_separator + 1);
		if((ar_custom_x > 0) && (ar_custom_y > 0)){
			int ar = scale_value * ar_custom_x / ar_custom_y;
			if((ar >= (scale_value / 100)) && (ar <= (scale_value * 100))) // FIXME: / 100
				ar_value = ar;
		}
	}
	return ar_value;
}

#define DISPQT_VIDEO_AUTOCROP_RGB32_LIMITMASK 0x00e0e0e0

bool FFMpegVideoWidget::videowidget_ffmpeg_swsctx_crop_auto_detect(uint8_t *video_data)
{
	const uint32_t limit_mask = DISPQT_VIDEO_AUTOCROP_RGB32_LIMITMASK;
	const int skip_border = DISPQT_CONFIG_VIDEO_AUTOCROP_SKIP_BORDER;
	const int check_range = 100 / DISPQT_CONFIG_VIDEO_AUTOCROP_CHECK_RANGE;
	const int width = this->sws_src_width, height = this->sws_src_height;
	int x, y, check_len, counter, crop_x = 0, crop_y = 0, crop_width = 0, crop_height = 0;
	uint32_t *data32_begin = (uint32_t *)video_data, *data32;
	bool retval = false;

	if(this->ffmpeg_swsframe_cropreset == DISPQT_VIDEO_AUTOCROP_COUNTER_OPEN){
		this->sws_srccrop_x = 0;
		this->sws_srccrop_y = 0;
		this->sws_srccrop_width = width;
		this->sws_srccrop_height = height;
		this->ffmpeg_swsframe_cropreset --;
		this->sws_autoxcrop_framecounter = 0;
		this->ffmpeg_swsframe_clearpic = true;
		retval = true;
		mpxplay_debugf(DISPQT_DEBUGOUT_ACROP, "AUTOCROP0 x:%d y:%d w:%d h:%d sw:%d sh:%d", crop_x, crop_y, crop_width, crop_height, this->sws_src_width, this->sws_src_height);
	}
	if((width < 32) || (height < 32))
		return false;
	if(this->sws_autoxcrop_framecounter > 0){
		this->sws_autoxcrop_framecounter--;
		return false;
	}

	// check top side
	x = skip_border;
	y = skip_border;
	data32 = data32_begin + y * width + x;
	check_len = height / check_range;
	do{
		if(*data32 & limit_mask){
			//mpxplay_debugf(DISPQT_DEBUGOUT_CROP, "auto1 data:%8.8X m:%8.8X x:%d y:%d", *data32, limit_mask, x,y);
			break;
		}
		data32++;
		if(++x >= (width - skip_border)){
			x = skip_border; y++;
			data32 = data32_begin + y * width + x;
		}
	}while(y < check_len);

	if(y >= check_len)  // probably full (or partially) black picture
		goto err_out_autocrop_black;
	if(y > skip_border)
		crop_y = y;

	// check bottom side
	x = skip_border;
	y = skip_border;
	data32 = data32_begin + (height - y) * width + x;
	check_len = height / check_range;
	do{
		if(*data32 & limit_mask){
			//mpxplay_debugf(DISPQT_DEBUGOUT_CROP, "auto1 data:%8.8X m:%8.8X c:%d", *data32, limit_mask, counter);
			break;
		}
		data32++;
		if(++x >= (width - skip_border)){
			x = skip_border; y++;
			data32 = data32_begin + (height - y) * width + x;
		}
	}while(y < check_len);

	if(y >= check_len)  // partially black picture
		goto err_out_autocrop_black;

	if(this->ffmpeg_swsframe_cropreset){ // top and bottom side crop shall be the same initially
		if(y > crop_y)
			y = crop_y;
		else if(crop_y > y)
			crop_y = y;
	}

	crop_height = height - crop_y;
	if((y > skip_border) && (y < check_len))
		crop_height -= y;

	//check left side
	x = skip_border;
	y = skip_border;
	data32 = data32_begin + y * width + x;
	check_len = width / check_range;
	do{
		if(*data32 & limit_mask){
			//mpxplay_debugf(DISPQT_DEBUGOUT_CROP, "auto3 data:%8.8X m:%8.8X x:%d", *data32, limit_mask, x);
			break;
		}
		data32 += width;
		if(++y >= (height - skip_border)){
			x++; y = skip_border;
			data32 = data32_begin + y * width + x;
		}
	}while(x < check_len);

	if(x >= check_len) // partially black picture
		goto err_out_autocrop_black;
	if((x > skip_border) && (x < check_len))
		crop_x = x - 1;

	//check right side
	x = skip_border;
	y = skip_border;
	data32 = data32_begin + y * width + width - x;
	check_len = width / check_range;
	do{
		if(*data32 & limit_mask)
			break;
		data32 += width;
		if(++y >= (height - skip_border)){
			x++; y = skip_border;
			data32 = data32_begin + y * width + width - x;
		}
	}while(x < check_len);

	if(x >= check_len) // partially black picture
		goto err_out_autocrop_black;

	if(this->ffmpeg_swsframe_cropreset){ // left and right side crop shall be the same initially
		if(x > crop_x)
			x = crop_x;
		else if(crop_x > x)
			crop_x = x;
	}

	crop_width = width - crop_x;
	if((x > skip_border) && (x < check_len))
		crop_width -= x - 1;

	if(this->ffmpeg_swsframe_cropreset){
		this->sws_srccrop_x = crop_x;
		this->sws_srccrop_y = crop_y;
		this->sws_srccrop_width = crop_width;
		this->sws_srccrop_height = crop_height;
		this->ffmpeg_swsframe_cropreset --;
		this->ffmpeg_swsframe_clearpic = true;
		mpxplay_debugf(DISPQT_DEBUGOUT_ACROP, "AUTOCROP1 x:%d y:%d w:%d h:%d sw:%d sh:%d", crop_x, crop_y, crop_width, crop_height, this->sws_src_width, this->sws_src_height);
		retval = true;
	}else{
		const int diff_change = DISPQT_CONFIG_VIDEO_AUTOCROP_SENSITIVITY;
		const int change_step_x = DISPQT_CONFIG_VIDEO_AUTOCROP_STEP, change_step_y = DISPQT_CONFIG_VIDEO_AUTOCROP_STEP;
		int change_step_w, change_step_h;

		change_step_w = change_step_x * 2;
		change_step_h = change_step_y * 2;

		if((crop_x + diff_change ) < this->sws_srccrop_x){
			this->sws_srccrop_x -= change_step_x;  retval = true;
		}else if(crop_x > (this->sws_srccrop_x + diff_change)){
			this->sws_srccrop_x += change_step_x;  retval = true;
		}else{
			change_step_w = change_step_x;
		}
		if((crop_y + diff_change ) < this->sws_srccrop_y){
			this->sws_srccrop_y -= change_step_y;  retval = true;
		}else if(crop_y > (this->sws_srccrop_y + diff_change)){
			this->sws_srccrop_y += change_step_y;  retval = true;
		}else{
			change_step_h = change_step_y;
		}
		if((crop_width + diff_change ) < this->sws_srccrop_width){
			this->sws_srccrop_width -= change_step_w;  retval = true;
		}else if(crop_width > (this->sws_srccrop_width + diff_change)){
			this->sws_srccrop_width += change_step_w;  retval = true;
		}
		if((crop_height + diff_change ) < this->sws_srccrop_height){
			this->sws_srccrop_height -= change_step_h;  retval = true;
		}else if(crop_height > (this->sws_srccrop_height + diff_change)){
			this->sws_srccrop_height += change_step_h;  retval = true;
		}

		mpxplay_debugf(DISPQT_DEBUGOUT_ACROP, "AUTOCROP2 r:%d CR:%d x:%d y:%d w:%d h:%d sw:%d sw:%d cx:%d cy:%d cw:%d ch:%d sx:%d sy:%d",
				(int)retval, this->ffmpeg_swsframe_cropreset, crop_x, crop_y, crop_width, crop_height, this->sws_src_width, this->sws_src_height,
				this->sws_srccrop_x, this->sws_srccrop_y, this->sws_srccrop_width, this->sws_srccrop_height, change_step_x, change_step_y);
		this->sws_autoxcrop_framecounter = (retval)? DISPQT_CONFIG_VIDEO_AUTOCROP_SHORTDELAY : DISPQT_CONFIG_VIDEO_AUTOCROP_LONGDELAY;
	}

	return retval;

err_out_autocrop_black:
	this->sws_autoxcrop_framecounter = DISPQT_CONFIG_VIDEO_AUTOCROP_SKIPDELAY;
	return retval;
}

bool FFMpegVideoWidget::videowidget_ffmpeg_swsctx_calculate(int window_width, int window_height)
{
	struct mmc_dispqt_config_s *gcfg;
	int video_ar, video_crop_ar = 0, window_ar, widget_global_pos_x, widget_global_pos_y;
	unsigned long ar_type;

	if((this->sws_src_width <= 0) || (this->sws_src_height <= 0) || (this->sws_src_pix_fmt <= AV_PIX_FMT_NONE))
		return false;

	this->sws_dst_width = window_width & DISPQT_VIDEO_ROUND_MASK;
	this->sws_dst_height = window_height & DISPQT_VIDEO_ROUND_MASK;

	if((window_width <= 0) || (window_height <= 0))
		return false;

	gcfg = this->main_window->gui_config;

	video_ar = DISPQT_VIDEO_AR_SCALE;
	video_ar *= this->sws_src_width * ((this->video_aspect_ratio.num > 0)? this->video_aspect_ratio.num : 1);
	video_ar /= this->sws_src_height * ((this->video_aspect_ratio.den > 0)? this->video_aspect_ratio.den : 1);

	// --- crop ratio calculate (manual, auto) -------------------------------------------------------------------------------------

	if((gcfg->video_crop_type != DISPQT_VIDEOSCRCROPTYPE_AUTO) || !this->sws_srccrop_width || !this->sws_srccrop_height
	 || (this->sws_srccrop_width > this->sws_src_width) || (this->sws_srccrop_height > this->sws_src_height)
	){
		this->sws_srccrop_x = this->sws_srccrop_y = 0;
		this->sws_srccrop_width = this->sws_src_width;
		this->sws_srccrop_height = this->sws_src_height;
	}

	switch(gcfg->video_crop_type)
	{
		case DISPQT_VIDEOSCRCROPTYPE_AUTO:  video_ar = video_ar * this->sws_srccrop_width / this->sws_src_width * this->sws_src_height / this->sws_srccrop_height; break;
		case DISPQT_VIDEOSCRCROPTYPE_4_3:   video_crop_ar = DISPQT_VIDEO_AR_SCALE * 4 / 3; break;
		case DISPQT_VIDEOSCRCROPTYPE_16_9:  video_crop_ar = DISPQT_VIDEO_AR_SCALE * 16 / 9; break;
		case DISPQT_VIDEOSCRCROPTYPE_235_1: video_crop_ar = DISPQT_VIDEO_AR_SCALE * 235 / 100; break;
		case DISPQT_VIDEOSCRCROPTYPE_CUSTOM:video_crop_ar = videowidget_ffmpeg_swsctx_get_custom_ar(gcfg->video_crop_custom, DISPQT_VIDEO_AR_SCALE, 0); break;
	}

	if((video_crop_ar > 0) && (video_crop_ar != video_ar))
	{
		if(video_ar > video_crop_ar)
			this->sws_srccrop_width = this->sws_src_width * video_crop_ar / video_ar;
		else
			this->sws_srccrop_height = this->sws_src_height * video_ar / video_crop_ar;
		this->sws_srccrop_x = (this->sws_src_width - this->sws_srccrop_width) / 2;
		this->sws_srccrop_y = (this->sws_src_height - this->sws_srccrop_height) / 2;
		mpxplay_debugf(DISPQT_DEBUGOUT_CROP, "CROP var:%d car:%d sw:%d sh:%d cw:%d ch:%d", video_ar, video_crop_ar, this->sws_src_width, this->sws_src_height, this->sws_srccrop_width, this->sws_srccrop_height);
		video_ar = video_crop_ar;
	}

	// --- aspect ratio calculate -------------------------------------------------------------------------------------

	ar_type = (this->isFullScreen() || (gcfg->video_windowed_ar_type == DISPQT_VIDEOWINDOWEDARTYPE_EQFULLSCREEN))? gcfg->video_fullscreen_ar_type : gcfg->video_windowed_ar_type;

	window_ar = DISPQT_VIDEO_AR_SCALE * window_width / window_height;

	switch(ar_type){
		case DISPQT_VIDEOFULLSCREENARTYPE_FILL:  video_ar = window_ar; break;
		case DISPQT_VIDEOFULLSCREENARTYPE_1_1:   video_ar = DISPQT_VIDEO_AR_SCALE; break;
		case DISPQT_VIDEOFULLSCREENARTYPE_4_3:   video_ar = DISPQT_VIDEO_AR_SCALE * 4 / 3; break;
		case DISPQT_VIDEOFULLSCREENARTYPE_16_9:  video_ar = DISPQT_VIDEO_AR_SCALE * 16 / 9; break;
		case DISPQT_VIDEOFULLSCREENARTYPE_235_1: video_ar = DISPQT_VIDEO_AR_SCALE * 235 / 100; break;
		case DISPQT_VIDEOFULLSCREENARTYPE_CUSTOM:video_ar = videowidget_ffmpeg_swsctx_get_custom_ar(gcfg->video_ar_custom, DISPQT_VIDEO_AR_SCALE, DISPQT_VIDEO_AR_SCALE); break;
		case DISPQT_VIDEOFULLSCREENARTYPE_PIXEL: video_ar = DISPQT_VIDEO_AR_SCALE * this->sws_src_width / this->sws_src_height; break;
	}

	// --- zoom value calculate -----------------------------------------------------------------------------------------

	const int prev_crop_w = this->sws_crop_w;
	const int prev_crop_h = this->sws_crop_h;
	const int pixel_ar = DISPQT_VIDEO_AR_SCALE * this->sws_srccrop_width / this->sws_srccrop_height;

	window_width = this->sws_srccrop_width;
	window_height = this->sws_srccrop_height;

	if(video_ar > pixel_ar)
		window_height = window_height * pixel_ar / video_ar;
	else if((video_ar > 0) && (video_ar < pixel_ar))
		window_width = window_width * video_ar / pixel_ar;

	if((window_width <= 0) || (window_height <= 0))
		return false;

	const int zoom_val_ti_w = (DISPQT_CONFIG_VIDEOZOOM_SCALE * this->sws_dst_width + (window_width >> 1)) / window_width;
	const int zoom_val_ti_h = (DISPQT_CONFIG_VIDEOZOOM_SCALE * this->sws_dst_height + (window_height >> 1)) / window_height;
	const int zoom_val_to = max(zoom_val_ti_w , zoom_val_ti_h);
	const int zoom_val_ti = (ar_type == DISPQT_VIDEOFULLSCREENARTYPE_FILL)? zoom_val_to : min(zoom_val_ti_w , zoom_val_ti_h);
	int zoom_value = DISPQT_CONFIG_VIDEOZOOM_CENTER;

	this->last_video_zoom_ti_val = zoom_val_ti;
//	this->last_video_zoom_min_val = ((ar_type == DISPQT_VIDEOFULLSCREENARTYPE_FILL) || (zoom_val_ti < DISPQT_CONFIG_VIDEOZOOM_MIN))? zoom_val_ti : DISPQT_CONFIG_VIDEOZOOM_MIN; // FIXME: if video resolution is larger than window, zoom is incorrect in fill mode below zoom_val_ti
	this->last_video_zoom_min_val = (zoom_val_ti < DISPQT_CONFIG_VIDEOZOOM_MIN)? zoom_val_ti : DISPQT_CONFIG_VIDEOZOOM_MIN;

	switch(gcfg->video_zoom_type)
	{
		case DISPQT_VIDEOFULLSCREENZOOMTYPE_TOUCHINSIDE: zoom_value = zoom_val_ti; break;
		case DISPQT_VIDEOFULLSCREENZOOMTYPE_TOUCHOUTSIDE: zoom_value = zoom_val_to; break;
		case DISPQT_VIDEOFULLSCREENZOOMTYPE_2_1: zoom_value = 2 * DISPQT_CONFIG_VIDEOZOOM_SCALE; break;
		case DISPQT_VIDEOFULLSCREENZOOMTYPE_4_1: zoom_value = 4 * DISPQT_CONFIG_VIDEOZOOM_SCALE; break;
		case DISPQT_VIDEOFULLSCREENZOOMTYPE_CUSTOM: zoom_value = videowidget_ffmpeg_swsctx_get_custom_ar(gcfg->video_zoom_custom, DISPQT_CONFIG_VIDEOZOOM_SCALE, DISPQT_CONFIG_VIDEOZOOM_SCALE); break;
		case DISPQT_VIDEOFULLSCREENZOOMTYPE_MANUAL: zoom_value = gcfg->video_zoom_value; break;
	}

	if(zoom_value < this->last_video_zoom_min_val)
	{
		zoom_value = this->last_video_zoom_min_val;
	}
	else if(zoom_value > DISPQT_CONFIG_VIDEOZOOM_MAX)
	{
		zoom_value = DISPQT_CONFIG_VIDEOZOOM_MAX;
	}

	if(zoom_value != DISPQT_CONFIG_VIDEOZOOM_CENTER){
		window_width = window_width * zoom_value / DISPQT_CONFIG_VIDEOZOOM_SCALE;
		window_height = window_height * zoom_value / DISPQT_CONFIG_VIDEOZOOM_SCALE;
	}

	//window_width = (window_width + DISPQT_VIDEO_ROUND_VALUE) & DISPQT_VIDEO_ROUND_MASK;
	//window_height = (window_height + DISPQT_VIDEO_ROUND_VALUE) & DISPQT_VIDEO_ROUND_MASK;

	this->sws_crop_w = this->sws_srccrop_width;
	this->sws_crop_h = this->sws_srccrop_height;

	if(window_width > this->sws_dst_width){
		this->sws_crop_w = this->sws_srccrop_width * this->sws_dst_width / window_width;
		window_width = this->sws_dst_width;
	}
	if(window_height > this->sws_dst_height){
		this->sws_crop_h = this->sws_srccrop_height * this->sws_dst_height / window_height;
		window_height = this->sws_dst_height;
	}

	//this->sws_crop_w = (this->sws_crop_w + DISPQT_VIDEO_ROUND_VALUE) & DISPQT_VIDEO_ROUND_MASK;
	//this->sws_crop_h = (this->sws_crop_h + DISPQT_VIDEO_ROUND_VALUE) & DISPQT_VIDEO_ROUND_MASK;

	gcfg->video_zoom_value = zoom_value;

	// --- zoom center (source view window) calculate -----------------------------------------------------------------------------------------

	if(this->isFullScreen()){
		widget_global_pos_x = widget_global_pos_y = 0;
	}else{
		widget_global_pos_x = QWidget::mapToGlobal(this->pos()).x();
		widget_global_pos_y = QWidget::mapToGlobal(this->pos()).y();
	}

	int max_crop_x = this->sws_srccrop_width - this->sws_crop_w;
	int max_crop_y = this->sws_srccrop_height - this->sws_crop_h;

	if( (gcfg->video_zoom_type == DISPQT_VIDEOFULLSCREENZOOMTYPE_TOUCHINSIDE) || (gcfg->video_zoom_type == DISPQT_VIDEOFULLSCREENZOOMTYPE_TOUCHOUTSIDE)){
		this->video_zoom_globalpos_x = widget_global_pos_x + this->sws_dst_width / 2;
		this->video_zoom_globalpos_y = widget_global_pos_y + this->sws_dst_height / 2;
		this->sws_crop_x = this->sws_srccrop_x + max_crop_x / 2;
		this->sws_crop_y = this->sws_srccrop_y + max_crop_y / 2;
	}
	else // 1:1, 2:1, 4:1, custom, manual
	{
		int zoom_widget_pos_x = this->video_zoom_globalpos_x - widget_global_pos_x;

		if((zoom_widget_pos_x >= 0) && (zoom_widget_pos_x < this->sws_dst_width))
		{
			if(prev_crop_w > 0){
				int sws_prev_pos_x = this->sws_crop_x + prev_crop_w * zoom_widget_pos_x / this->sws_dst_width;
				mpxplay_debugf(DISPQT_DEBUGOUT_FFMPEG, "ZOOMX px:%d", sws_prev_pos_x);
				this->sws_crop_x = sws_prev_pos_x - this->sws_crop_w * zoom_widget_pos_x / this->sws_dst_width;
			}else{
				this->sws_crop_x = this->sws_srccrop_x + max_crop_x * zoom_widget_pos_x / this->sws_dst_width;
			}
			if(this->sws_crop_x < this->sws_srccrop_x)
				this->sws_crop_x = this->sws_srccrop_x;
			else if(this->sws_crop_x > (this->sws_srccrop_x + max_crop_x))
				this->sws_crop_x = this->sws_srccrop_x + max_crop_x;
		}else{
			this->sws_crop_x = this->sws_srccrop_x + max_crop_x / 2;
		}

		int zoom_widget_pos_y = this->video_zoom_globalpos_y - widget_global_pos_y;

		if((zoom_widget_pos_y >= 0) && (zoom_widget_pos_y < this->sws_dst_height))
		{
			if(prev_crop_h > 0){
				int sws_prev_pos_y = this->sws_crop_y + prev_crop_h * zoom_widget_pos_y / this->sws_dst_height;
				mpxplay_debugf(DISPQT_DEBUGOUT_FFMPEG, "ZOOMX px:%d", sws_prev_pos_y);
				this->sws_crop_y = sws_prev_pos_y - this->sws_crop_h * zoom_widget_pos_y / this->sws_dst_height;
			}else{
				this->sws_crop_y = this->sws_srccrop_y + max_crop_y * zoom_widget_pos_y / this->sws_dst_height;
			}
			if(this->sws_crop_y < this->sws_srccrop_y)
				this->sws_crop_y = this->sws_srccrop_y;
			else if(this->sws_crop_y > (this->sws_srccrop_y + max_crop_y))
				this->sws_crop_y = this->sws_srccrop_y + max_crop_y;
		}else{
			this->sws_crop_y = this->sws_srccrop_y + max_crop_y / 2;
		}

		if(zoom_value == zoom_val_ti)
			gcfg->video_zoom_type = DISPQT_VIDEOFULLSCREENZOOMTYPE_TOUCHINSIDE;
		else if(zoom_value == zoom_val_to)
			gcfg->video_zoom_type = DISPQT_VIDEOFULLSCREENZOOMTYPE_TOUCHOUTSIDE;
		else if(zoom_value == DISPQT_CONFIG_VIDEOZOOM_CENTER)
			gcfg->video_zoom_type = DISPQT_VIDEOFULLSCREENZOOMTYPE_1_1;

		mpxplay_debugf(DISPQT_DEBUGOUT_FFMPEG, "ZOOM cx:%d cy:%d", this->sws_crop_x, this->sws_crop_y);
	}

	mpxplay_debugf(DISPQT_DEBUGOUT_CROP, "sws_calculate war:%d var:%d n:%d d:%d sw:%d sh:%d ww:%d wh:%d z:%d zw:%d zh:%d cx:%d cy:%d cw:%d ch:%d",
			window_ar, video_ar, this->video_aspect_ratio.num, this->video_aspect_ratio.den, this->sws_src_width, this->sws_src_height,
			this->sws_dst_width, this->sws_dst_height, zoom_value, zoom_val_ti_w, zoom_val_ti_h, this->sws_crop_x, this->sws_crop_y, this->sws_crop_w, this->sws_crop_h);

	// --- target window calculate -----------------------------------------------------------------------------------------

	if(window_width < this->sws_dst_width)
	{
		this->sws_dst_pos_x = (this->sws_dst_width - window_width) / 2;
		this->sws_dst_width = window_width;
		this->ffmpeg_swsframe_clearpic = true;
	}else{
		this->sws_dst_pos_x = 0;
	}
	if(window_height < this->sws_dst_height)
	{
		this->sws_dst_pos_y = (this->sws_dst_height - window_height) / 2;
		this->sws_dst_height = window_height;
		this->ffmpeg_swsframe_clearpic = true;
	}else{
		this->sws_dst_pos_y = 0;
	}

	return true;
}
//-------------------------------------------------------------------------------------------------------------------------------------

int FFMpegVideoWidget::videowidget_ffmpeg_swsctx_init(int window_width, int window_height)
{
	enum AVPixelFormat output_pixelformat = AV_PIX_FMT_RGB32;
	int numBytes;

	//mpxplay_debugf(DISPQT_DEBUGOUT_FFMPEG, "videowidget_ffmpeg_swsctx_init BEGIN (close)");
	this->videowidget_ffmpeg_swsctx_close();

	if(!this->videowidget_ffmpeg_swsctx_calculate(window_width, window_height))
		goto err_out_init;

	this->ffmpeg_sws_frame = av_frame_alloc();
	if(!this->ffmpeg_sws_frame)
		goto err_out_init;

#ifdef USE_SWS_IMAGE_SCALING
	numBytes = av_image_get_buffer_size(output_pixelformat, window_width, window_height, 1);
#else
	numBytes = av_image_get_buffer_size(output_pixelformat, this->sws_src_width, this->sws_src_height, 1);
#endif
	if(!numBytes)
		goto err_out_init;
	this->sws_buffer = (uint8_t *)av_malloc(numBytes * sizeof(uint8_t));
	if(!this->sws_buffer)
		goto err_out_init;
#ifdef USE_SWS_IMAGE_SCALING
	if(av_image_fill_arrays(&this->ffmpeg_sws_frame->data[0], &this->ffmpeg_sws_frame->linesize[0], this->sws_buffer, output_pixelformat, window_width, window_height, 1) <= 0)
		goto err_out_init;
	mpxplay_debugf(DISPQT_DEBUGOUT_FFMPEG, "sws_getContext BEGIN sw:%d sh:%d sp:%d ww:%d wh:%d op:%d", this->sws_src_width, this->sws_src_height, (int)this->sws_src_pix_fmt,
			window_width, window_height, (int)output_pixelformat);
	this->sws_ctx = sws_getContext(this->sws_src_width, this->sws_src_height, this->sws_src_pix_fmt,
									window_width, window_height, output_pixelformat, SWS_BILINEAR, NULL, NULL, NULL);
#else
	if(av_image_fill_arrays(&this->ffmpeg_sws_frame->data[0], &this->ffmpeg_sws_frame->linesize[0], this->sws_buffer, output_pixelformat, this->sws_src_width, this->sws_src_height, 1) <= 0)
		goto err_out_init;
//	mpxplay_debugf(DISPQT_DEBUGOUT_FFMPEG, "sws_getContext BEGIN sw:%d sh:%d sp:%d ww:%d wh:%d op:%d", this->sws_src_width, this->sws_src_height, (int)this->sws_src_pix_fmt,
//			window_width, window_height, (int)output_pixelformat);
	this->sws_ctx = sws_getContext(this->sws_src_width, this->sws_src_height, this->sws_src_pix_fmt,
			this->sws_src_width, this->sws_src_height, output_pixelformat, SWS_POINT, NULL, NULL, NULL);

#endif
	//mpxplay_debugf(DISPQT_DEBUGOUT_FFMPEG, "sws_getContext END ctx:%8.8X", (int)this->sws_ctx);
	if(!this->sws_ctx)
		goto err_out_init;
	//mpxplay_debugf(DISPQT_DEBUGOUT_FFMPEG, "videowidget_ffmpeg_swsctx_init w:%d h:%d b:%d", window_width, window_height, numBytes);
	return MPXPLAY_ERROR_OK;

err_out_init:
	this->sws_src_width = 0;
	this->sws_src_height = 0;
	this->sws_src_pix_fmt = AV_PIX_FMT_NONE;
	return -1;
}

void FFMpegVideoWidget::videowidget_ffmpeg_swsctx_close(void)
{
	if(this->ffmpeg_sws_frame)
		av_frame_free(&this->ffmpeg_sws_frame);
	if(this->sws_buffer)
		av_free(this->sws_buffer);
	if(this->sws_ctx)
		sws_freeContext(this->sws_ctx);
	if(this->sub_title_infos){
		video_ffmpeg_subtitleinfo_clear(this->sub_title_infos);
		av_freep(&this->sub_title_infos);
	}
	this->videowidget_ffmpeg_swsctx_clear();
}

void FFMpegVideoWidget::videowidget_ffmpeg_swsctx_clear(void)
{
	this->ffmpeg_sws_frame = NULL;
	this->ffmpeg_swsframe_hasdata = false;
	this->ffmpeg_swsframe_clearpic = true;
	this->sws_buffer = NULL;
	this->sws_ctx = NULL;
	this->ffmpeg_swsframe_cropreset = DISPQT_VIDEO_AUTOCROP_COUNTER_OPEN;
	this->sws_autoxcrop_framecounter = 0;
	this->sub_title_infos = NULL;
}

bool FFMpegVideoWidget::videowidget_ffmpeg_config_callback(unsigned int cfgcmd, mpxplay_infile_callback_t *cbk_func, void *passdata)
{
	bool is_video_open = false;
	//mpxplay_debugf(DISPQT_DEBUGOUT_FFMPEG, "videowidget_ffmpeg_config_callback %8.8X %8.8X", (int)cbk_func, (int)passdata);
	this->mutex_video.tryLock(DISPQT_VIDEO_MUTEX_TIMEOUT);
	if(this->video_decoder_worker)
		is_video_open = this->video_decoder_worker->videothread_ffmpeg_config_callback(cfgcmd, cbk_func, passdata);
	this->mutex_video.unlock();
	if(this->video_decoder_worker && (is_video_open == false))
		this->videowidget_ffmpeg_frame_display(-1, 0, NULL, NULL);
	return is_video_open;
}

int FFMpegVideoWidget::videowidget_ffmpeg_callback_call(int globalpos_x, int globalpos_y, unsigned long command, void *arg1, int *arg2)
{
	int retval = -1;
	this->mutex_video.tryLock(DISPQT_VIDEO_MUTEX_TIMEOUT);
	if(this->video_decoder_worker)
		retval = this->video_decoder_worker->videothread_ffmpeg_callback_call(globalpos_x, globalpos_y, command, arg1, arg2);
	this->mutex_video.unlock();
	return retval;
}

void FFMpegVideoWidget::videowidget_ffmpeg_frame_display(int video_index, unsigned int flags, AVFrame *videoout_frame, ffmpegvideo_subtitle_info_s *subtitle_infos)
{
	struct mmc_dispqt_config_s *gcfg = this->main_window->gui_config;
	bool doSwsInit = false, doSwsScale = false, doRepaint = false;

	this->mutex_video.tryLock(DISPQT_VIDEO_MUTEX_TIMEOUT);

#ifdef USE_SWS_IMAGE_SCALING
	av_frame_unref(this->ffmpeg_videoout_frame);
#endif
	if(video_index < 0)// means close rendering
	{
		if(this->ffmpeg_sws_frame)
			doRepaint = true;
		this->videowidget_ffmpeg_swsctx_close();
		mpxplay_debugf(DISPQT_DEBUGOUT_FFMPEG, "videowidget_ffmpeg_packet_store CLOSE");
	}
	else if(flags & DISPQT_FFMPGVIDEOCALLBACKFLAG_FIRSTFRAME) // means new file
	{
		this->ffmpeg_swsframe_cropreset = DISPQT_VIDEO_AUTOCROP_COUNTER_OPEN;
		this->sws_autoxcrop_framecounter = 0;
		mpxplay_debugf(DISPQT_DEBUGOUT_ACROP, "videowidget_ffmpeg_frame_display RESET2");
	}
	else if(!videoout_frame) // means seeking
	{
		if(!this->ffmpeg_swsframe_cropreset){
			this->ffmpeg_swsframe_cropreset = DISPQT_VIDEO_AUTOCROP_COUNTER_REFRESH;
			mpxplay_debugf(DISPQT_DEBUGOUT_ACROP, "videowidget_ffmpeg_frame_display RESET1");
		}
		this->sws_autoxcrop_framecounter = 0;
	}

	if(videoout_frame)
	{
		if(!this->sws_ctx
		 || ((flags & DISPQT_FFMPGVIDEOCALLBACKFLAG_STILLPICT) && ((this->sws_src_width < videoout_frame->width) || (this->sws_src_height < videoout_frame->height))) // display the largest frame at still pictures (eg. progressive jpeg)
		 || (!(flags & DISPQT_FFMPGVIDEOCALLBACKFLAG_STILLPICT) && ((this->sws_src_width != videoout_frame->width) || (this->sws_src_height != videoout_frame->height)))
		 || (this->sws_src_pix_fmt != (AVPixelFormat)videoout_frame->format)
		 || (this->video_aspect_ratio.num != videoout_frame->sample_aspect_ratio.num) || (this->video_aspect_ratio.den != videoout_frame->sample_aspect_ratio.den)
		){
			this->sws_src_width = videoout_frame->width;
			this->sws_src_height = videoout_frame->height;
			this->sws_src_pix_fmt = (AVPixelFormat)videoout_frame->format;
			this->video_aspect_ratio.num = videoout_frame->sample_aspect_ratio.num;
			this->video_aspect_ratio.den = videoout_frame->sample_aspect_ratio.den;
			mpxplay_debugf(DISPQT_DEBUGOUT_FFMPEG, "WHP change w:%d h:%d p:%d arw:%d arh:%d", width, height, (int)pixfmt, ar_w, ar_h);
			doSwsInit = true;
			doSwsScale = true;
		}
		else if(!(flags & DISPQT_FFMPGVIDEOCALLBACKFLAG_STILLPICT)) // not progressive jpeg
		{
			doSwsScale = true;
		}

		if(this->sub_title_infos)
		{
			video_ffmpeg_subtitleinfo_clear(this->sub_title_infos);
		}
		if(subtitle_infos)
		{
			if(!this->sub_title_infos)
				this->sub_title_infos = (ffmpegvideo_subtitle_info_s *)av_mallocz(sizeof(ffmpegvideo_subtitle_info_s));
			video_ffmpeg_subtitleinfo_copy(this->sub_title_infos, subtitle_infos);
		}
	}

	if(doSwsInit)
		videowidget_ffmpeg_swsctx_init(this->width(), this->height());

	if(doSwsScale && this->sws_ctx){
		//mpxplay_debugf(DISPQT_DEBUGOUT_FFMPEG, "sws_scale BEGIN ctx:%8.8X", this->sws_ctx);
		if(sws_scale(this->sws_ctx, (uint8_t const * const *)&videoout_frame->data[0],
				videoout_frame->linesize, 0, this->sws_src_height,
				&this->ffmpeg_sws_frame->data[0], &this->ffmpeg_sws_frame->linesize[0]) ==
#ifdef USE_SWS_IMAGE_SCALING
				this->sws_dst_height
#else
				this->sws_src_height
#endif
		){
			if(gcfg->video_crop_type == DISPQT_VIDEOSCRCROPTYPE_AUTO)
				if(this->videowidget_ffmpeg_swsctx_crop_auto_detect(this->ffmpeg_sws_frame->data[0]))
					this->videowidget_ffmpeg_swsctx_calculate(this->width(), this->height());
			this->ffmpeg_swsframe_hasdata = true;
			doRepaint = true;
			//mpxplay_debugf(DISPQT_DEBUGOUT_FFMPEG, "sws_scale d:%8.8X l:%d", this->ffmpeg_sws_frame->data[0], this->ffmpeg_sws_frame->linesize[0]);
		}
	}

	if(videoout_frame)
		av_frame_free(&videoout_frame);

	this->mutex_video.unlock();

	if(doRepaint){
		if(this->main_window->gui_config->video_renderer_type == DISPQT_VIDEORENDERTYPE_WIDGET)
		{
			this->repaint();
		}
		else
		{
			// TODO
		}
	}
}

void FFMpegVideoWidget::setFullScreen(bool fullScreen){}
void FFMpegVideoWidget::setAspectRatioMode(Qt::AspectRatioMode mode){}
int FFMpegVideoWidget::brightness() const { return 100; };
int FFMpegVideoWidget::contrast() const { return 100; };
int FFMpegVideoWidget::hue() const { return 100; };
int FFMpegVideoWidget::saturation() const { return 100; };

void FFMpegVideoWidget::setBrightness(int brightness) {}
void FFMpegVideoWidget::setContrast(int contrast) {}
void FFMpegVideoWidget::setHue(int hue) {}
void FFMpegVideoWidget::setSaturation(int saturation) {}

#endif // MPXPLAY_LINK_ORIGINAL_FFMPEG
